﻿#!/usr/bin/env python
# -*- Python -*-

import sys
import time
sys.path.append(".")

# Import RTM module
import OpenRTM
import RTC

from math import *
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
import pygame
from pygame.locals import *
import numpy
import codecs
import os

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
GLRTM_spec = ["implementation_id", "GLRTM", 
		 "type_name",         "GLRTM", 
		 "description",       "GL RTM Component", 
		 "version",           "0.1", 
		 "vendor",            "Kyouzyu", 
		 "category",          "TEST", 
		 "activity_type",     "DataFlowComponent", 
		 "max_instance",      "1", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 ""]
# </rtc-template>

class GLRTM(OpenRTM.DataFlowComponentBase):
	def __init__(self, manager):
		OpenRTM.DataFlowComponentBase.__init__(self, manager)

		self._d_Rot = RTC.TimedDoubleSeq(RTC.Time(0,0),[])
		self._RotIn = OpenRTM.InPort("Rot", self._d_Rot, OpenRTM.RingBuffer(8))
		self._d_Pos = RTC.TimedDoubleSeq(RTC.Time(0,0),[])
		self._PosIn = OpenRTM.InPort("Pos", self._d_Pos, OpenRTM.RingBuffer(8))
		

		# Set InPort buffers
		self.registerInPort("Rot",self._RotIn)
		self.registerInPort("Pos",self._PosIn)
		self.Func = GL_Function()
                
		
		# Set OutPort buffers
		

		

		

		
		# Set service provider to Ports
		
		# Set service consumers to Ports
		
		# Set CORBA Service Ports
		

		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">
		
		# </rtc-template>


		 
	def onInitialize(self):
		# Bind variables and configuration variable
		
		return RTC.RTC_OK


	
	#def onFinalize(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onStartup(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onShutdown(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	def onActivated(self, ec_id):
            pygame.init()
            self.Func.init_GL()
	
	    return RTC.RTC_OK
	
	def onDeactivated(self, ec_id):
            pygame.quit()
	
	    return RTC.RTC_OK
	
	def onExecute(self, ec_id):
            if self._RotIn.isNew():
                inport = self._RotIn.read()
                j = -1
                for i in range(0, len(inport.data)):
                    if i % 9 == 0:
                        j += 1
                    self.Func.bodies[j].rot[i-j*9] = inport.data[i]
            if self._PosIn.isNew():
                inport = self._PosIn.read()
                j = -1
                for i in range(0, len(inport.data)):
                    if i % 3 == 0:
                        j += 1
                    if i % 3 == 0:
                        self.Func.bodies[j].px = inport.data[i]
                    if i % 3 == 1:
                        self.Func.bodies[j].py = inport.data[i]
                    if i % 3 == 2:
                        self.Func.bodies[j].pz = inport.data[i]
            self.Func._drawfunc()
            
            pygame.display.flip()

            for event in pygame.event.get():
                pass
            
	    return RTC.RTC_OK
	
	#def onAborting(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onError(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onReset(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onStateUpdate(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onRateChanged(self, ec_id):
	#
	#	return RTC.RTC_OK
	

def prepare_GL(window_size):
 
    w, h = window_size
    glClearColor(0.8,0.8,0.9,0)
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST)
    glDisable(GL_LIGHTING)
    glEnable(GL_LIGHTING)
    glEnable(GL_NORMALIZE)
    glShadeModel(GL_FLAT)
    

    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(60.0, float(w) / float(h), 1.0, 100.0)

    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

    glLightfv(GL_LIGHT0,GL_POSITION,[0,0,1,0])
    glLightfv(GL_LIGHT0,GL_DIFFUSE,[1,1,1,1])
    glLightfv(GL_LIGHT0,GL_SPECULAR,[1,1,1,1])
    glEnable(GL_LIGHT0)

    gluLookAt (4.4, 0.0, 2.0, 0.0, 0.0, 1.0, 0, 0, 1)

def myCylinder(r, h, dn):
    PI2 = 3.141592*2
    step = PI2 / 25.0
    if dn == 1:
        glNormal3d(-1.0, 0.0, 0.0)
        glBegin(GL_TRIANGLE_FAN)
        for i in range(0, 25):
            t = step * i
            glVertex3d(h, r * sin(t), r * cos(t))
        glEnd()
        glNormal3d(1.0, 0.0, 0.0);
        glBegin(GL_TRIANGLE_FAN);
        for i in range(0, 25):
            t = step * (25 - i)
            glVertex3d(h, r * sin(t), r * cos(t))
        glEnd()

        glBegin(GL_QUAD_STRIP)
        for i in range(0, 25):
            t = step * i;
            z = sin(t);
            x = cos(t);

            glNormal3d(0.0, x, z)
            glVertex3f(h, r * x, r * z)
            glVertex3f(0.0, r * x, r * z)
        
        glEnd()
    elif dn == 2:
        glNormal3d(0.0, 1.0, 0.0)
        glBegin(GL_TRIANGLE_FAN)
        for i in range(0, 25):
            t = step * i
            glVertex3d(r * sin(t), h, r * cos(t))
        glEnd()
        glNormal3d(0.0, -1.0, 0.0)
        glBegin(GL_TRIANGLE_FAN)
        for i in range(0, 25):
            t = step * (25 - i)
            glVertex3d(r * sin(t), h, r * cos(t))
        glEnd()

        glBegin(GL_QUAD_STRIP)
        for i in range(0, 25):
            t = step * i
            x = sin(t)
            z = cos(t)

            glNormal3d(x, 0.0, z)
            glVertex3f(r * x, h, r * z)
            glVertex3f(r * x, 0.0, r * z)
        
        glEnd();
    elif dn == 3:
        glNormal3d(0.0, 0.0, -1.0)
        glBegin(GL_TRIANGLE_FAN)
        for i in range(0, 25):
            t = step * i
            glVertex3d(r * sin(t), r * cos(t), h)
        glEnd()
        glNormal3d(0.0, 0.0, 1.0)
        glBegin(GL_TRIANGLE_FAN)
        for i in range(0, 25):
            t = step * (25 - i)
            glVertex3d(r * sin(t), r * cos(t), h)
        glEnd()

        glBegin(GL_QUAD_STRIP)
        for i in range(0, 25):
            t = step * i
            z = sin(t)
            x = cos(t)

            glNormal3d(x, z, 0.0)
            glVertex3f(r * x, r * z, h)
            glVertex3f(r * x, r * z, 0.0)
        
        glEnd()
def myCapsule(r, h, dn):
    myCylinder(r, h, dn)
    if dn == 1:
        x = h
        y = 0
        z = 0
    elif dn == 2:
        x = 0
        y = h
        z = 0
    elif dn == 3:
        x = 0
        y = 0
        z = h
        
    glTranslatef(x, y, z)
    glutSolidSphere(r, 10, 10)
    
    glTranslatef(-x, -y, -z)
    glutSolidSphere(r, 10, 10)

class GL_Function:
    def __init__(self):
        self.bodynum = 0
        self.bodies = []
        
    
    def draw_body(self):
        green = [0.0, 1.0, 0.0, 1.0]
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, green)
        for i in range(0, self.bodynum):
            x = self.bodies[i].px
            y = self.bodies[i].py
            z = self.bodies[i].pz
            R = self.bodies[i].rot
            rot = [R[0], R[3], R[6], 0.,
                   R[1], R[4], R[7], 0.,
                   R[2], R[5], R[8], 0.,
                   x, y, z, 1.0]
            glPushMatrix()
            glMultMatrixd(rot)
            if self.bodies[i].shape == "box":
                sx = self.bodies[i].lx
                sy = self.bodies[i].ly
                sz = self.bodies[i].lz
                glScale(sx, sy, sz)
                glutSolidCube(1)
            elif self.bodies[i].shape == "sphere":
                sr = self.bodies[i].r
                glutSolidSphere(sr, 10, 10)
            elif self.bodies[i].shape == "cylinder":
                myCylinder(self.bodies[i].r, self.bodies[i].l, self.bodies[i].dn)
            elif self.bodies[i].shape == "capsule":
                myCapsule(self.bodies[i].r, self.bodies[i].l, self.bodies[i].dn)

            glPopMatrix()

    def draw_ground(self):
        black = [0.2, 0.2, 0.2, 1.0]
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, black)
        glPushMatrix()
        sx,sy,sz = [100, 100, 1]
        glScale(sx, sy, sz)
        glTranslatef(0.0, 0.0, -0.5) 
        glutSolidCube(1)
        glPopMatrix()
    

    def create_box(self, lx, ly, lz, px, py, pz, Mass):
        link = BODY()
        link.lx = lx
        link.ly = ly
        link.lz = lz
        link.px = px
        link.py = py
        link.pz = pz
        link.shape = "box"

        self.bodies.append(link)
        self.bodynum += 1
        
    def create_sphere(self, r, px, py, pz, Mass):
        link = BODY()
        link.r = r
        link.px = px
        link.py = py
        link.pz = pz
        link.shape = "sphere"

        self.bodies.append(link)
        self.bodynum += 1

    def create_cylinder(self, dn,  l, r ,px, py, pz, Mass):
        link = BODY()
        link.l = l
        link.r = r
        link.px = px
        link.py = py
        link.pz = pz
        link.dn = dn
        link.shape = "cylinder"
        self.bodies.append(link)
        self.bodynum += 1
        
    def create_capsule(self, dn,  l, r, px, py, pz,Mass):
        link = BODY()
        link.l = l
        link.r = r
        link.px = px
        link.py = py
        link.pz = pz
        link.dn = dn
        link.shape = "capsule"
        self.bodies.append(link)
        self.bodynum += 1
        


    def glpygameinit(self, window_size):
       w, h = window_size
       glClearColor(0.8,0.8,0.9,0)
       glEnable(GL_DEPTH_TEST)
       glViewport(0, 0, w, h)
       glMatrixMode(GL_PROJECTION)
       glLoadIdentity()
       gluPerspective(60.0, float(w) / float(h), 1.0, 100.0)
       glMatrixMode(GL_MODELVIEW)
        
    def init_GL(self):
    
                
        #self.init_force_data(2)
        """glutInit ([])

        glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE)

        x = 0
        y = 0
        width = 640
        height = 480
        glutInitWindowPosition (x, y);
        glutInitWindowSize (width, height);
        glutCreateWindow ("test")"""
        window_size = (800, 600)
        screen = pygame.display.set_mode(
            window_size,
            OPENGL | DOUBLEBUF,
            )
        pygame.display.set_caption(u'テスト'.encode('utf-8'))
        self.glpygameinit(window_size)


        glEnable(GL_CULL_FACE)
        glCullFace(GL_BACK)

        self.create_box(0.2, 0.7, 0.2, 0.0, 0.0, 0.2, 10.0)

    def _drawfunc (self):
        
        prepare_GL((800, 600))
        self.draw_body()
        self.draw_ground()
        glFlush()
        
        
    def _idlefunc (self):
        glutPostRedisplay ()

        
    def _keyfunc (self, key, x, y):
        ESCAPE = 27
        if key == ESCAPE:
            sys.exit (0)


class BODY:
    def __init__(self):
        self.px = 0.
        self.py = 0.
        self.pz = 0.
        self.rot = [1., 0., 0., 0., 1., 0., 0., 0., 1.]
        
def MyModuleInit(manager):
    profile = OpenRTM.Properties(defaults_str=GLRTM_spec)
    manager.registerFactory(profile,
                            GLRTM,
                            OpenRTM.Delete)

    # Create a component
    comp = manager.createComponent("GLRTM")



def main():
	mgr = OpenRTM.Manager.init(len(sys.argv), sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

